import { rm, green, white, bold, ms, run, resolveBin } from '@genesislcap/build-kit';
import type { BuildContext } from '@genesislcap/build-kit';
import consola from 'consola';
import { rollup, watch } from 'rollup';
import { getInputOptions, getOutputOptions } from './options';

export default async (ctx: BuildContext) => {
  const {
    dirs: { cwd },
    cli: { isDev, isAnalyze, isBuild, options },
    env: { isCI },
    config: { externals, clean, copyAssets, entry },
    pkg: { nameUnscoped },
  } = ctx;

  if (clean) {
    await rm(['dist', '.rollup.cache']);
  }

  if (Array.isArray(copyAssets)) {
    const copyfiles = await resolveBin('copyfiles');
    copyAssets.forEach((glob) => {
      run(cwd, `${copyfiles} --verbose --up 1 src/${glob} ./dist/esm`);
    });
  }

  const start = Date.now();
  const openStats = options.open && !isCI;
  const input = getInputOptions(externals, isAnalyze, openStats, entry);
  const output = getOutputOptions(externals, isAnalyze, isDev, nameUnscoped);
  const files = output.map((o) => o.file);
  consola.log(white(`\n${bold(input.input)} → ${bold(files.join(', '))}...`));

  if (isDev) {
    const watchOptions = {
      ...input,
      output,
    };
    const watcher = watch(watchOptions);
    watcher.on('event', (event) => {
      if (event.code === 'BUNDLE_END') {
        event.result.close();
      }
    });
  } else if (isAnalyze || isBuild) {
    const bundle = await rollup(input);
    await Promise.all(output.map(bundle.write));
    await bundle.close();
    consola.success(
      green(
        `bundled ${bold(bundle.watchFiles.length)} files → ${bold(files.join(', '))} in ${bold(
          ms(Date.now() - start),
        )}`,
      ),
    );
  } else {
    throw new Error(`Unrecognized command: ${JSON.stringify(options)}`);
  }
};
